// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import { n } from './eccParams';
import BaseScalar from './baseScalar';
import BigInt from 'big-integer';

export default class EcScalar extends BaseScalar {
    /**
     * @param {(bigInt.BigInteger|number|string)} value
     */
    constructor(value) {
        super(value, n);
    }

    static get zero() { return new EcScalar(BigInt.zero) }
    
    static get one() { return new EcScalar(BigInt.one) }

    /**
     * @returns {Uint8Array}
     */
    toArray() {
        var bytes = super.toArray();
        return bytes.length >= 32 ? bytes : Buffer.concat([new Uint8Array(32 - bytes.length), bytes], 32);
    }

    /**
     * @param {Uint8Array|number[]} data
     * @returns {EcScalar}
     */
    static fromArray(data) {
        return Object.setPrototypeOf(super._fromArray(data, n), EcScalar.prototype);
    }

    /**
     * @returns {EcScalar}
     */
    static random() {
        return Object.setPrototypeOf(super._random(n), EcScalar.prototype);
    }
}
