import BigInt from 'big-integer';
import EcScalar from '../ecScalar';
import ByteSerializer from '../byteSerializer';

export default class TFastRj {
    /**
     * @param {bigInt.BigInteger} tranId
     * @param {boolean} isFromNeg
     * @param {EcScalar} from
     * @param {EcScalar} to
     * @param {EcScalar} ciuirjsj
     * @param {EcScalar} vi
     * @param {Uint8Array} sign
     */
    constructor(tranId, isFromNeg, from, to, ciuirjsj, vi, sign) {
        this.tranId = tranId;
        this.isFromNeg = isFromNeg;
        this.from = from;
        this.to = to;
        this.ciuirjsj = ciuirjsj;
        this.vi = vi;
        this.sign = sign;
    }

    toString() {
        return Buffer.from(this.toBytearray()).toString("base64");
    }

    toBytearray() {
        var tranId = new Uint8Array(this.tranId.toArray(256).value);
        var fromSign = new Uint8Array(this.from.value.shiftLeft(1).or(this.isFromNeg ? 1 : 0).toArray(256).value)

        return ByteSerializer.Serialize([tranId, fromSign, this.to.toArray(), this.ciuirjsj.toArray(), this.vi.toArray(), this.sign], 1);
    }

    /**
     * @param {string | ArrayBuffer | number[]} data
     */
    static from(data) {
        var buffer = typeof data === 'string' ? Buffer.from(data, "base64")
            : data instanceof ArrayBuffer ? Buffer.from(data) : Buffer.from(data);

        var nums = ByteSerializer.deserializeBytes(buffer, 1);
        if (nums.length !== 6)
            throw new Error("Format error");

        var tranId = BigInt.fromArray(Array.from(nums[0]), 256, false);
        var fromSign = BigInt.fromArray(Array.from(nums[1]), 256, false);
        var isFromNeg = fromSign.and(1).greater(0);
        var from = new EcScalar(fromSign.shiftRight(1));

        return new TFastRj(tranId, isFromNeg, from, EcScalar.fromArray(nums[2]), EcScalar.fromArray(nums[3]), EcScalar.fromArray(nums[4]), nums[5]);
    }
}
